# Copyright (c) 2018 Kevin Weiss, for HAW Hamburg  <kevin.weiss@haw-hamburg.de>
#
# This file is subject to the terms and conditions of the MIT License. See the
# file LICENSE in the top level directory for more details.
# SPDX-License-Identifier:    MIT
"""packet init for RIOT PAL
This exposes usful modules in the RIOT PAL packet
"""
from pathlib import Path
from .dut_shell import DutShell
from .ll_shell import LLShell
from .ll_mem_map_if import LLMemMapIf

PHILIP_MEM_MAP_PATH = str(Path(__file__).parents[0]) + \
                      '/mem_map/philip_mem_map-00_00_01.csv'
__all__ = ['DutShell', 'LLShell', 'LLMemMapIf']
